unit convertlogfileunit;

{$mode objfpc}

interface

uses
  appsettings, //Required to read application settings (like locations).
  dateutils, //Required to convert logged UTC string to TDateTime
  strutils, //Required for checking lines in conversion file.
  moon, //required for Moon calculations
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type

  { Tconvertdialog }

  Tconvertdialog = class(TForm)
    CloseButton: TButton;
    OutputFilenameDisplay: TLabeledEdit;
    LatitudeDisplay: TLabeledEdit;
    LongitudeDisplay: TLabeledEdit;
    Memo1: TMemo;
    OpenFileDialog: TOpenDialog;
    SelectButton: TButton;
    StatusBar1: TStatusBar;
    procedure CloseButtonClick(Sender: TObject);
    procedure SelectButtonClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  convertdialog: Tconvertdialog;
  MyLatitude: extended = 0.0; //Latitude
  MyLongitude: extended = 0.0; //Longitude
  MyElevation: extended = 0.0; //Elevation

implementation

{ Tconvertdialog }

procedure Tconvertdialog.SelectButtonClick(Sender: TObject);
var
    File1,OutFile: TextFile;
    Str: String;
    pieces: TStringList;

    MoonElevation: extended = 0.0;
    MoonAzimuth: extended = 0.0;

    UTCRecord :TDateTime;
    ComposeString: String;
    OutFileString: String;

    WriteAllowable: Boolean = True; //Allow output file to be written or not.

begin
     pieces := TStringList.Create;

     OpenFileDialog.Filter:='data log files|*.dat|All files|*.*';
     OpenFileDialog.InitialDir := appsettings.LogsDirectory;
if OpenFileDialog.Execute then
  begin
    //Start reading file.
    AssignFile(File1, OpenFileDialog.Filename);
    OutFileString:=ChangeFileExt(OpenFileDialog.Filename,'.csv');
    AssignFile(OutFile, OutFileString);
    OutputFilenameDisplay.Text:=OutFileString;
    if FileExists(OutFileString) then
      begin
        if (MessageDlg('Overwrite existing file?','Do you want to overwrite the existing file?',mtConfirmation,[mbOK,mbCancel],0) = mrOK) then
            WriteAllowable:=True
        else
          WriteAllowable:=False;
      end;
    if WriteAllowable then
      begin
    {$I+}
    try
      Reset(File1);

      Rewrite(OutFile); //Open file for writing

      repeat
        // Read one line at a time from the file.
        Readln(File1, Str);

        // Get location data from header.
        if AnsiStartsStr('# Position',Str) then
          begin
            //Prepare for parsing.
            pieces.Delimiter := ',';
            pieces.StrictDelimiter := True; //Do not parse spaces.
            //Remove comment from beginning of line.
            pieces.DelimitedText := AnsiRightStr(Str,length(Str) - RPos(':',Str));
            if (pieces.Count<2) then
              begin
                MessageDlg('Error', 'No location data found in header.', mtError, [mbOK],0);
                Writeln(OutFile,'Error: No location data found in header.');
                LatitudeDisplay.Text:='nul';
                LongitudeDisplay.Text:='nul';
                break;
              end
            else
              begin
               MyLatitude:=StrToFloat(pieces.Strings[0]);
               if ((MyLatitude<-90.0) or (MyLatitude>90.0)) then
                 begin
                   writeln('Error: Latitude ',MyLatitude,' out of range');
                   break;
                 end;
               MyLongitude:=StrToFloat(pieces.Strings[1]);
               if ((MyLongitude<-180.0) or (MyLongitude>180.0)) then
                 begin
                   writeln('Error: Longitude ',MyLongitude,' out of range');
                   break;
                 end;
                LatitudeDisplay.Text:=Format('%.3f',[MyLatitude]);
                LongitudeDisplay.Text:=Format('%.3f',[MyLongitude]);
              end;
          end;

        //Get and modify record header string
        if AnsiStartsStr('# YYYY-MM-DDTHH:mm:ss.fff;',Str) then
          Writeln(OutFile,'UTC '+AnsiRightStr(Str,length(Str) - 2)+';MoonPhaseDeg;MoonElevDeg;MoonIllum%');

        //Ignore comment lines which have # as first character.
        if not AnsiStartsStr('#',Str) then
          begin
            //Separate the fields of the record.
            pieces.Delimiter := ';';
            pieces.DelimitedText := Str;

            //Make sure there are enough fields
            if ((pieces.Count<5) or (pieces.Count>6)) then
              begin
                MessageDlg('Error', 'Incorrect number of fields in record.', mtError, [mbOK],0);
                break;
              end
            else
            begin

            //parse the fields, and convert as necessary.
            //Convert UTC string 'YYYY-MM-DDTHH:mm:ss.fff' into TDateTime
            UTCRecord:=ScanDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz',pieces.Strings[0]);
            //writeln(FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz',UTCRecord));

            //Calculate Moon position
            //Change sign for Moon calculations
            Moon_Position_Horizontal(
                                     StrToDateTime(DateTimeToStr(UTCRecord)),
                                     -1.0*MyLongitude,
                                     MyLatitude,
                                     MoonElevation,
                                     MoonAzimuth);

            //Prepare string for output.
            ComposeString:= Str + ';'
                           //Moon Phase angle (0 to 180 degrees).
                           + Format('%.1f;',[moon_phase_angle(StrToDateTime(DateTimeToStr(UTCRecord)))])

                           //Moon elevation (positive = above horizon, negative = below horizon).
                           + Format('%.3f;',[MoonElevation])

                           //Moon illumination pecent.
                           + Format('%.1f;',[current_phase(StrToDateTime(DateTimeToStr(UTCRecord)))*100.0])
            ;

            WriteLn(OutFile,ComposeString);
            end;//End of checking number of fields in record.

          end;
      until(EOF(File1)); // EOF(End Of File) The the program will keep reading new lines until there is none.
      CloseFile(File1);
    except
      on E: EInOutError do
      begin
       MessageDlg('Error', 'File handling error occurred. Details: '+E.ClassName+'/'+E.Message, mtError, [mbOK],0);
      end;
    end;
    Flush(OutFile);
    CloseFile(OutFile);
            end;//End of WriteAllowable check.

  end;

end;

procedure Tconvertdialog.CloseButtonClick(Sender: TObject);
begin
  Close;
end;


initialization
  {$I convertlogfileunit.lrs}

end.

