unit dlretrieve;

{$mode objfpc}

interface

uses
  Classes, SysUtils, FileUtil, SynMemo, TATools, TASources, TAGraph,
  TAFuncSeries, TASeries, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, Buttons, LCLType, ExtCtrls, ComCtrls
 ,UMathInterpolation
 ,StrUtils
 ;


type

  { TDLRetrieveForm }

  TDLRetrieveForm = class(TForm)
    Button1: TButton;
    Chart2: TChart;
    Chart2ColorMapSeries1: TColorMapSeries;
    Chart2LineSeries1: TLineSeries;
    EastLabel: TLabel;
    ExportButton: TButton;
    Chart1: TChart;
    Chart1ColorMapSeries1: TColorMapSeries;
    Chart1LineSeries1: TLineSeries;
    Chart1LineSeries2: TLineSeries;
    ChartToolset1: TChartToolset;
    ChartToolset1PanDragTool1: TPanDragTool;
    ChartToolset1ZoomDragTool1: TZoomDragTool;
    CheckBox1: TCheckBox;
    DLCancelRetrieveButton: TButton;
    DLGHeaderButton: TButton;
    DLRetrieveAllButton: TButton;
    Edit1: TEdit;
    Edit2: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    ListChartSource1: TListChartSource;
    ListChartSource2: TListChartSource;
    NorthLabel: TLabel;
    OpenDialog2: TOpenDialog;
    OpenAnotherFileButton: TBitBtn;
    PageControl1: TPageControl;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    RecentFileEdit: TLabeledEdit;
    OpenDialog1: TOpenDialog;
    OpenDirectoryButton: TBitBtn;
    OpenRecentFileButton: TBitBtn;
    FileDirectoryEdit: TLabeledEdit;
    ShowDotsCheckBox: TCheckBox;
    ShowLinesCheckBox: TCheckBox;
    SouthLabel: TLabel;
    StaticText1: TStaticText;
    SynMemo1: TSynMemo;
    SynMemo2: TSynMemo;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    WestLabel: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure Chart1ColorMapSeries1Calculate(const AX, AY: Double; out
      AZ: Double);
    procedure Chart1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer
      );
    procedure Chart2ColorMapSeries1Calculate(const AX, AY: Double; out
      AZ: Double);
    procedure CheckBox1Change(Sender: TObject);
    procedure DLCancelRetrieveButtonClick(Sender: TObject);
    procedure DLGHeaderButtonClick(Sender: TObject);
    procedure DLRetrieveAllButtonClick(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure Edit2Change(Sender: TObject);
    procedure ExportButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OpenAnotherFileButtonClick(Sender: TObject);
    procedure OpenDirectoryButtonClick(Sender: TObject);
    procedure OpenRecentFileButtonClick(Sender: TObject);
    procedure ShowLinesCheckBoxChange(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  DLRetrieveForm: TDLRetrieveForm;

  //FormContourmap: TFormContourmap;
  Int3D: TInterpolation3D;

implementation

uses
  Unit1, math,appsettings
  ,header_utils,dateutils
  ,dlheader //contains the DLHeader code
  ,dlerase
  ,vector
  ;

{ TDLRetrieveForm }

procedure TDLRetrieveForm.DLGHeaderButtonClick(Sender: TObject);
begin
  DLHeaderForm.ShowModal;
end;

procedure TDLRetrieveForm.DLCancelRetrieveButtonClick(Sender: TObject);
begin
  DLCancelRetrieve:=True;
end;

procedure TDLRetrieveForm.Chart1MouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var x1,y1,z1: Double;
begin
  If Chart1LineSeries1.IsEmpty then Exit;
  x1 := Chart1.XImageToGraph(X);
  y1 := Chart1.YImageToGraph(Y);
  z1 := Int3D.GetIntData(x1,y1);
  StaticText1.Caption :=
    'X: ' + FloatToStrF(x1,ffFixed,6,3) + #13 +
    'Y: ' + FloatToStrF(y1,ffFixed,6,3) + #13 +
    'Z: ' + FloatToStrF(z1,ffFixed,6,3);
end;

procedure TDLRetrieveForm.Chart2ColorMapSeries1Calculate(const AX, AY: Double;
  out AZ: Double);
begin
  AZ:=AY;
end;

procedure TDLRetrieveForm.CheckBox1Change(Sender: TObject);
begin
  Chart1LineSeries1.Marks.Visible := CheckBox1.Checked;
end;

procedure TDLRetrieveForm.Button1Click(Sender: TObject);
var i: Integer;
    SL: TStringList;

    Darkness, Altitude, Azimuth, xValue, yValue: Double;
    pieces: TStringList; //delimited result from generic read

begin

    SL := TStringList.Create;
    Int3D.ClearData;

    Chart1LineSeries1.BeginUpdate;
    Chart1LineSeries1.Clear;
    Int3D.InterpolationMode := IM_DELAUNAY;

    pieces := TStringList.Create;
    pieces.Delimiter := ';';
    pieces.StrictDelimiter := False; //Parse spaces also

    //SL.LoadFromFile('DataUDMtest.dat'); // Quick-and-Dirty data import
    OpenDialog1.Execute;
    SL.LoadFromFile(OpenDialog1.FileName); // Quick-and-Dirty data import

    For i:=1 {without header line} to SL.Count-1 do begin

      pieces.DelimitedText := SL.Strings[i];
      if pieces.Count=14 then begin
        Darkness:=StrToFloatDef(pieces[4],0);
        Altitude:=StrToFloatDef(pieces[11],0);
        Azimuth:=StrToFloatDef(pieces[12],0);
        xValue:=cos(degtorad(Azimuth+90))*(1.0-abs(Altitude/90.0));
        yValue:=sin(degtorad(Azimuth+90))*(1.0-abs(Altitude/90.0));
        Int3D.AddData(
          xValue,
          yValue,
          Darkness
          );
        Chart1LineSeries1.AddXY(xValue,yValue, FloatToStr(Altitude)+' '+FloatToStr(Azimuth)); // point number as Mark-Label
      end
      else
          writeln('pcs=',pieces.Count,' string= ',SL.Strings[i]);
    end;

  Chart1LineSeries2.BeginUpdate;
  Chart1LineSeries2.Clear;
  For i:=0 to High(Int3D.Triangles) do begin
    Chart1LineSeries2.AddXY(Int3D.Data[Int3D.Triangles[i].a,0],Int3D.Data[Int3D.Triangles[i].a,1]);
    Chart1LineSeries2.AddXY(Int3D.Data[Int3D.Triangles[i].b,0],Int3D.Data[Int3D.Triangles[i].b,1]);
    Chart1LineSeries2.AddXY(Int3D.Data[Int3D.Triangles[i].c,0],Int3D.Data[Int3D.Triangles[i].c,1]);
    Chart1LineSeries2.AddXY(Int3D.Data[Int3D.Triangles[i].a,0],Int3D.Data[Int3D.Triangles[i].a,1]);
    Chart1LineSeries2.AddXY(NaN,NaN); // manual breakpoint (TAChart backward compartible) or AddNull if possible
  end;
  Chart1LineSeries2.EndUpdate;


  //MPSAS fixed values
  Int3D.zmin:=10.0;
  Int3D.zmax:=21.0;

  Edit1.Text := FloatToStr(Int3D.zmin); // start values for manual scaling
  Edit2.Text := FloatToStr(Int3D.zmax);

  SL.Free;
end;

procedure TDLRetrieveForm.Chart1ColorMapSeries1Calculate(const AX, AY: Double;
  out AZ: Double);
begin
    AZ := Int3D.GetNomalizedIntData(AX,AY);
end;

procedure TDLRetrieveForm.DLRetrieveAllButtonClick(Sender: TObject);
var
  result: String;
  subfix: AnsiString;
  pieces: TStringList;
  LogTime : TDateTime;
  Temperature, Voltage, Darkness : Float;
  TriggerModeNumber: Integer;
  DesiredPieces: Integer;
begin
  pieces := TStringList.Create;
  pieces.Delimiter := ',';

  SynMemo2.Clear;
  RecentFileEdit.Text:='';

  //Ensure that continuous logging mode has not been selected because it interferes with data retrieval on firmware less than feature 30.
  result:=sendget('Lmx');
  TriggerModeNumber:=StrToInt(AnsiMidStr(result,4,1));

  //Warn user that erase cannot be done
  if ( (StrToIntDef(SelectedFeature,0)<30) and ((TriggerModeNumber=1) or (TriggerModeNumber=2))) then begin
    Application.MessageBox('Cannot retrieve database while in continuous trigger mode.'+ sLineBreak+
    'Select trigger mode = Off,'+ sLineBreak+
    ' or one of the "Every x on the hour" modes.', 'Busy logging, cannot retrieve!', MB_ICONEXCLAMATION)  ;
    exit;
  end;

   if ((Length(DLHeaderForm.TZRegionBox.Text)>0) and (Length(DLHeaderForm.TZLocationBox.Text)>0)) then
     begin
     DLCancelRetrieve:=False;
       //Get unit time for comparison
       try
          if (SelectedModel=model_V) then
            WriteDLHeader('DL-V-Log','DL Retrieve All') //vector model
          else
            WriteDLHeader('DL-Log','DL Retrieve All');// Standard DL model

          //Set the desired pieces to be seen when getting a record
          // There are two extra pieces than normal since the date record is split up
          if SelectedModel=model_V then
             DesiredPieces:=14    //vector model
          else
             DesiredPieces:=7;    //other DL models

          for DLCurrentRecord:=1 to DLEStoredRecords do begin
              if DLCancelRetrieve then begin
                 StatusMessage('DL Retrieve cancelled.');
                 break;
               end;
               pieces.DelimitedText := sendget(Format('L4%010.10dx',[DLCurrentRecord-1]));
               pieces.StrictDelimiter := False; //Parse spaces also because date is split up

               SynMemo1.Lines.Clear;
               if (pieces.Count=DesiredPieces) then begin
                 SynMemo1.Lines.Append(Format('Retrieved record: %d / %d',[DLCurrentRecord, DLEStoredRecords]));
                 //writeln(Format('Retrieved record: %d / %d',[DLCurrentRecord, DLEStoredRecords]));//debug
                 Application.ProcessMessages;

                 { Pull in unit time for UTC/Local calculations. }
                 try
                    LogTime:=ScanDateTime('yy-mm-ddhh:nn:ss',pieces.Strings[1]+pieces.Strings[3]);
                 except
                    LogTime:=ScanDateTime('yy-mm-dd hh:nn:ss','01-01-01 01:01:01');//Record something anyway even if local date is messed up
                 end;

                 { Pull in values }
                 Temperature:=StrToFloatDef(AnsiLeftStr(pieces.Strings[5],Length(pieces.Strings[5])-1),0,FPointSeparator);
                 Voltage:=(2.048 + (3.3 * StrToFloatDef(pieces.Strings[6],0,FPointSeparator))/256.0);
                 Darkness:=StrToFloatDef(pieces.Strings[4],0,FPointSeparator);

                 { Write record to logfile }
                 Write(DLRecFile,
                    FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"',LogTime) + //Date UTC
                    FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"',dlheader.ptz.GMTToLocalTime(LogTime,DLHeaderForm.TZLocationBox.Text,subfix)) + //Date Local (calculated)
                    FormatFloat('##0.0',Temperature,FPointSeparator) + ';' + //Temperature
                    FormatFloat('0.00',Voltage,FPointSeparator) + ';' + //Voltage
                    FormatFloat('#0.00',Darkness,FPointSeparator) //mpsas value
                 );

                 if SelectedModel=model_V then begin
                   Write(DLRecFile,Format(';%d;',[StrToIntDef(pieces.Strings[7],0)]));//Acceleration X
                   Write(DLRecFile,Format('%d;',[StrToIntDef(pieces.Strings[8],0)]));//Acceleration Y
                   Write(DLRecFile,Format('%d;',[StrToIntDef(pieces.Strings[9],0)]));//Acceleration Z
                   Write(DLRecFile,Format('%d;',[StrToIntDef(pieces.Strings[10],0)]));//Magnetic X
                   Write(DLRecFile,Format('%d;',[StrToIntDef(pieces.Strings[11],0)]));//Magnetic Y
                   Write(DLRecFile,Format('%d;',[StrToIntDef(pieces.Strings[12],0)]));//Magnetic Z
                   Ax:= -1.0 * StrToFloatDef(pieces.Strings[7],0);
                   Ay:=        StrToFloatDef(pieces.Strings[8],0);
                   Az:=        StrToFloatDef(pieces.Strings[9],0);
                   NormalizeAccel(); //Compute acceleration values (In the future, this may be done inside the PIC)
                   Write(DLRecFile,Format('%0.0f;',[radtodeg(arcsin(-1.0*Ax1))]));
                   //
                   Mx:=        StrToFloatDef(pieces.Strings[10],0);
                   My:= -1.0 * StrToFloatDef(pieces.Strings[11],0);
                   Mz:= -1.0 * StrToFloatDef(pieces.Strings[12],0);
                   NormalizeMag();
                   ComputeAzimuth();
                   Heading:=radtodeg(arctan2(-1*Mz2,Mx2))+180;
                   Write(DLRecFile,Format('%0.0f;',[Heading]));
                   Write(DLRecFile,Format('%d',[StrToIntDef(pieces.Strings[13],0)]));//Vibration count
                 end;

                 writeln(DLRecFile,'');//Finish writing line to logfile
               end
               else begin
                  StatusMessage(Format('Failed: actual count = %d , desired count = %d ',[pieces.Count, DesiredPieces]));
                  SynMemo1.Lines.Append(Format('Failed: actual count = %d , desired count = %d ',[pieces.Count, DesiredPieces]));
                  //break;
               end;
          end;
           SynMemo1.Lines.Clear;
           if DLCancelRetrieve then
              SynMemo1.Lines.Append('Partially retrieved records written to:')
           else
               SynMemo1.Lines.Append('Retrieved records written to:');
           RecentFileEdit.Text:=Format('%s',[LogFileName]);
           SynMemo1.Lines.Append(Format('%s',[LogFileName]));
           DLCancelRetrieve:=False;
       except
             StatusMessage('ERROR! IORESULT: ' + IntToStr(IOResult));
       end;
       Flush(DLRecFile);
       CloseFile(DLRecFile);
     end
   else
       ShowMessage('Enter Time zone information into Header first. '+ sLineBreak+
       'Do this by pressing the Header button, then selecting your timezone.');

end;

procedure TDLRetrieveForm.Edit1Change(Sender: TObject);
var tmp: Double;
begin
  If TryStrToFloat(Edit1.Text,tmp) then begin
    Int3D.zmin := tmp;
    Chart1.Invalidate;
  end;
end;

procedure TDLRetrieveForm.Edit2Change(Sender: TObject);
var tmp: Double;
begin
  If TryStrToFloat(Edit2.Text,tmp) then begin
    Int3D.zmax := tmp;
    Chart1.Invalidate;
  end;
end;

procedure TDLRetrieveForm.ExportButtonClick(Sender: TObject);
begin
  Chart1.SaveToFile(TJPEGImage, 'MyImage.jpg');
end;

procedure TDLRetrieveForm.FormCreate(Sender: TObject);
begin

  PageControl1.PageIndex:=0; //Always start with displaying the "Text" page

  With ListChartSource1 do begin
    Clear;
    Add(1,0,'',$000000);   // Black
    Add(0.6,0,'',$640054);   // Dark purple
    Add(0.3,0,'',$FF0000); // Blue
    Add(0.1,0,'',$FFFF00); // Turquise
    Add(0.0,0,'',$7FFF00);
    Add(-0.1,0,'',$00FF00);  // Gren
    Add(-0.3,0,'',$00FF7F);
    Add(-0.6,0,'',$00FFFF);  // Yellow
    Add(-0.9,0,'',$0000FF);  // Red
    Add(-1.0,0,'',$FFFFFF);  // White
  end;

  With ListChartSource2 do begin
    Clear;
    Add(22.0,0,'',$000000);  // Black
    Add(19.6,0,'',$640054);  // Dark purple
    Add(17.8,0,'',$FF0000);  // Blue
    Add(16.6,0,'',$FFFF00);  // Turquise
    Add(16.0,0,'',$7FFF00);
    Add(15.4,0,'',$00FF00);  // Green
    Add(14.2,0,'',$00FF7F);
    Add(12.4,0,'',$00FFFF);  // Yellow
    Add(10.6,0,'',$0000FF);  // Red
    Add(10.0,0,'',$FFFFFF);  // White
  end;

Chart2LineSeries1.AddXY(0.0,10.0);
Chart2LineSeries1.AddXY(0.0,22.0);

end;

procedure TDLRetrieveForm.FormShow(Sender: TObject);
begin
     FileDirectoryEdit.Text:=Format('%s%',[appsettings.LogsDirectory + DirectorySeparator]);
end;

procedure TDLRetrieveForm.OpenAnotherFileButtonClick(Sender: TObject);
var
  Filename: String;
begin
  OpenDialog1.Execute;

  Filename:=OpenDialog1.FileName;
  RecentFileEdit.Text:=Filename;

  if FileExists(Filename) then
    SynMemo2.Lines.LoadFromFile(Filename)
    else
       begin
        SynMemo2.Clear;
        SynMemo2.Lines.Append('File:');
        SynMemo2.Lines.Append(format('    %s',[Filename]));
        SynMemo2.Lines.Append('does not exist!');
      end;
end;

procedure TDLRetrieveForm.OpenDirectoryButtonClick(Sender: TObject);
begin
    OpenDialog1.InitialDir := appsettings.LogsDirectory;
  if OpenDialog1.Execute then
  begin{ TODO : file viewer form2, make something new maybe. }
    SynMemo2.Lines.LoadFromFile(OpenDialog1.Filename);
    //Form2.Memo1.Lines.LoadFromFile(OpenDialog1.Filename);
    //Form2.Show;
  end;

end;

procedure TDLRetrieveForm.OpenRecentFileButtonClick(Sender: TObject);
var
  RecentFilname: String;
begin

  RecentFilname:=RecentFileEdit.Text;

  if FileExists(RecentFilname) then
    SynMemo2.Lines.LoadFromFile(RecentFilname)
    else
       begin
        SynMemo2.Clear;
        SynMemo2.Lines.Append('File:');
        SynMemo2.Lines.Append(format('    %s',[RecentFilname]));
        SynMemo2.Lines.Append('does not exist!');
      end;
end;

procedure TDLRetrieveForm.ShowLinesCheckBoxChange(Sender: TObject);
begin
      Chart1LineSeries1.Pointer.Visible:=ShowDotsCheckBox.Checked;

      Chart1LineSeries2.BeginUpdate;
      Chart1LineSeries2.Clear;
      Chart1LineSeries2.EndUpdate;
end;

initialization
  {$I dlretrieve.lrs}
  Int3D := TInterpolation3D.Create;

Finalization
  If Int3D <> nil then Int3D.Free;

end.

