unit worldmap;

{$mode objfpc}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, StdCtrls;

type

  { TFormWorldmap }

  TFormWorldmap = class(TForm)
    ActualElevationEdit: TLabeledEdit;
    ActualLatitudeEdit: TLabeledEdit;
    ActualLongitudeEdit: TLabeledEdit;
    ApplyButton: TButton;
    CloseButton: TButton;
    CursorLatitude: TLabeledEdit;
    CursorLongitude: TLabeledEdit;
    DesiredElevationEdit: TLabeledEdit;
    DesiredLabel: TLabel;
    AppliedLabel: TLabel;
    DesiredLatitudeEdit: TLabeledEdit;
    DesiredLongitudeEdit: TLabeledEdit;
    MapImage: TImage;
    CreditLabel: TLabel;
    CursorLabel: TLabel;
    LatitudeLabel: TLabel;
    LongitudeLabel: TLabel;
    ElevationLabel: TLabel;
    procedure ApplyButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure DesiredElevationEditChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MapImageClick(Sender: TObject);
    procedure MapImageMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer
      );
    procedure DesiredLatitudeEditChange(Sender: TObject);
    procedure DesiredLongitudeEditChange(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

//Declare local procedures
procedure UpdateLocationLines();
procedure CheckApplyButton();

var
  FormWorldmap: TFormWorldmap;
  TempLat, TempLon: Extended;

implementation

uses
  convertlogfileunit
  ,appsettings //for file locations
  ,dlheader //contains the DLHeader code
  ;

const
  mapfile = 'world.topo.bathy.200412.3x800x400.jpg';

{ TFormWorldmap }

procedure TFormWorldmap.FormShow(Sender: TObject);
begin

  //Get lat/lon displayed on this window
  TempLat:=MyLatitude;
  TempLon:=MyLongitude;

  ActualLatitudeEdit.Text:=Format('%0.6f',[TempLat]);
  ActualLongitudeEdit.Text:=Format('%0.6f',[TempLon]);
  ActualElevationEdit.Text:=Format('%0.1f',[MyElevation]);

  DesiredLatitudeEdit.Text:=ActualLatitudeEdit.Text;
  DesiredLongitudeEdit.Text:=ActualLongitudeEdit.Text;
  DesiredElevationEdit.Text:=ActualElevationEdit.Text;

  UpdateLocationLines();

  ApplyButton.Enabled:=False;

end;

procedure TFormWorldmap.MapImageClick(Sender: TObject);
begin
  DesiredLatitudeEdit.Text:=CursorLatitude.Caption;
  DesiredLongitudeEdit.Text:=CursorLongitude.Caption;

  TempLat:=StrToFloatDef(DesiredLatitudeEdit.text,0);
  TempLon:=StrToFloatDef(DesiredLongitudeEdit.text,0);

  UpdateLocationLines();
  CheckApplyButton();

  //ApplyButton.Enabled:=True;
end;

procedure TFormWorldmap.ApplyButtonClick(Sender: TObject);
begin
  //Set global variables
  MyLatitude:=TempLat;
  MyLongitude:=TempLon;
  MyElevation:=StrToFloatDef(DesiredElevationEdit.Text,0);
  //Set displayed "Applied" text
  ActualLatitudeEdit.Text:=DesiredLatitudeEdit.Text;
  ActualLongitudeEdit.Text:=DesiredLongitudeEdit.Text;
  ActualElevationEdit.Text:=DesiredElevationEdit.Text;

  //Set position values on DL Header screen.
  //The change event there also writes to the ini file.
  DLHeaderForm.PositionEntry.Text:=
     ActualLatitudeEdit.Text + ', ' +
     ActualLongitudeEdit.Text + ', ' +
     ActualElevationEdit.Text;

  vConfigurations.WriteString(INISection,'Position',DLHeaderForm.PositionEntry.Text);


  Application.ProcessMessages;
  CheckApplyButton();
end;

procedure TFormWorldmap.CancelButtonClick(Sender: TObject);
begin
    Close;
end;

procedure TFormWorldmap.CloseButtonClick(Sender: TObject);
begin
    Close;
end;

procedure TFormWorldmap.DesiredElevationEditChange(Sender: TObject);
begin
  CheckApplyButton();
end;

procedure TFormWorldmap.DesiredLatitudeEditChange(Sender: TObject);
begin
end;

procedure TFormWorldmap.DesiredLongitudeEditChange(Sender: TObject);
begin
end;

procedure TFormWorldmap.MapImageMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  with FormWorldmap.MapImage do begin
       CursorLatitude.Caption:=format('%0.6f',[(Y - Height/2) * -90/(Height/2)]);
       CursorLongitude.Caption:=format('%0.6f',[(X - Width/2) * 180/(Width/2)]);
  end;
end;

procedure CheckApplyButton();
var
  Different: Boolean;
begin
  with FormWorldmap do begin
    if DesiredLatitudeEdit.Text <> ActualLatitudeEdit.Text then begin
      Different:=True;
      ActualLatitudeEdit.Font.Color:=clRed;
    end
    else
      ActualLatitudeEdit.Font.Color:=clDefault;

    if DesiredLongitudeEdit.Text <> ActualLongitudeEdit.Text then begin
      Different:=True;
      ActualLongitudeEdit.Font.Color:=clRed;
    end
    else
      ActualLongitudeEdit.Font.Color:=clDefault;

    if DesiredElevationEdit.Text <> ActualElevationEdit.Text then begin
      Different:=True;
      ActualElevationEdit.Font.Color:=clRed;
    end
    else
      ActualElevationEdit.Font.Color:=clDefault;

    ApplyButton.Enabled:=Different;

  end;
end;

procedure UpdateLocationLines();
var
  LatitudePostion, LongitudePostion : Integer;
begin

  with FormWorldmap.MapImage do begin
    Picture.LoadFromFile(appsettings.DataDirectory+mapfile);
    LatitudePostion:=round(Height/2 - (TempLat *  Height/2)/90 );
    LongitudePostion:=round(Width/2 + (TempLon *  Width/2)/180);

    Picture.Bitmap.Canvas.Pen.Color:=clWhite;
    Picture.Bitmap.Canvas.Line(0,LatitudePostion, Width,LatitudePostion);
    Picture.Bitmap.Canvas.Line(LongitudePostion,0, LongitudePostion,Height);
  end;

end;

initialization
  {$I worldmap.lrs}

end.

